@@ -7,7 +7,7 @@
 # Software Foundation; either version 2.1 of the License, or (at your option)
 # any later version.
 #
-# Paramiko is distrubuted in the hope that it will be useful, but WITHOUT ANY
+# Paramiko is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 # A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 # details.
@@ -24,6 +24,7 @@ from __future__ import generators
 
 import array
 from binascii import hexlify, unhexlify
+import errno
 import sys
 import struct
 import traceback
@@ -270,6 +271,14 @@ def get_logger(name):
     l.addFilter(_pfilter)
     return l
 
+def retry_on_signal(function):
+    """Retries function until it doesn't raise an EINTR error"""
+    while True:
+        try:
+            return function()
+        except EnvironmentError, e:
+            if e.errno != errno.EINTR:
+                raise
 
 class Counter (object):
     """Stateful counter for CTR mode crypto"""
